


#### DESCRIPTION OF CODE
## This code is based on code used by Pollesh et al. (2019)and was used to support the analyses of the manuscript entitled
## "Cross-species applicability of an adverse outcome pathway network for thyroid hormone system disruption"
## by Ann-Cathrin Haigis, Lucia Vergauwen, Carlie A. LaLone, Daniel L. Villeneuve, Jason M. O'Brien, Dries Knapen



#### OTHER FILES NEEDED ####
## An XML file from the AOPWiki,used below: aop-wiki-xml-2022-03-16.xml which is the Wiki download from March 16, 2022
## A list of selected AOPs,used below: thyroid-AOPN-AOP-IDs.txt which was manually curated on March 16, 2022
## The "Functions_THSD-AOPnetwork_Construction.R" file. This script should be run previous to the "THSD AOP network_tDOA.R" as this
## script draws functions from the "Functions_THSD-AOPnetwork_Construction.R" file.
## Cytoscape software which should be started previous to exporting th igraph object to Cytoscape

#### LIBRARIES AND SOURCE CODE #####

## Initialize ##
#Load packages#

library(xml2)
library(igraph)
library(RCy3)
library(RColorBrewer)

## set source directory ##
source("Functions_THSD-AOPnetwork_Construction.R")

#### load FILES ####
## load AOP WIKI download and list with IDs from AOPs of interest ##
aopXML<-"aop-wiki-xml-2022-03-16"
## Make sure the extension of the file is '.xml'
AOPs_of_interest<-unlist(read.table("thyroid-AOPN-AOP-IDs.txt", header = TRUE, sep="\t"), use.names = FALSE)

#### LOAD XML AND REF ID DATA ####

# IMPORT XML
xData<-read_xml(aopXML)
xData<-xml_ns_strip(xData)

# REF ID TO AOPWIKI ID
# link xml references of KEs, KERs, AOPs and taxa to their AOP ID (ID number)
keID<-data.frame(
  ref=xml_attr(xml_find_all(xData, "/data/vendor-specific/key-event-reference"),"id"),
  ID=as.numeric(xml_attr(xml_find_all(xData, "/data/vendor-specific/key-event-reference"),"aop-wiki-id")),
  stringsAsFactors=FALSE
)

kerID<-data.frame(
  ref=xml_attr(xml_find_all(xData, "/data/vendor-specific/key-event-relationship-reference"),"id"),
  ID=as.numeric(xml_attr(xml_find_all(xData, "/data/vendor-specific/key-event-relationship-reference"),"aop-wiki-id")),
  stringsAsFactors=FALSE
)

aopID<-data.frame(
  ref=xml_attr(xml_find_all(xData, "/data/vendor-specific/aop-reference"),"id"),
  ID=as.numeric(xml_attr(xml_find_all(xData, "/data/vendor-specific/aop-reference"),"aop-wiki-id")),
  stringsAsFactors=FALSE
)

taxID<-data.frame(
  ref=xml_attr(xml_find_all(xData, "/data/vendor-specific/taxonomy-reference"),"id"),
  ID=as.numeric(xml_attr(xml_find_all(xData, "/data/vendor-specific/taxonomy-reference"),"aop-wiki-id")),
  stringsAsFactors=FALSE
)

#### TAXONOMY DATA TABLE ####
## summarizes taxa specific identifiers from xml file in one data frame 
taxData<-data.frame(
  ID=taxID$ID[match(xml_attr(xml_find_all(xData, "/data/taxonomy"), "id"),taxID$ref)],
  name=xml_text(xml_find_all(xData, "/data/taxonomy/name")),
  source=xml_text(xml_find_all(xData, "/data/taxonomy/source")),
  source_ID=xml_text(xml_find_all(xData, "/data/taxonomy/source-id")),
  stringsAsFactors=FALSE
)


#### KEY EVENT (KE) DATA TABLE ####

# KE taxonomy data (lists all species to which a KE is assigned in the AOP Wiki and respective WoE calls)
keTaxDom<-lapply(xml_find_all(xData, "/data/key-event/applicability"),FUN=function(x){
  if("taxonomy"%in%xml_name(xml_children(x))){
    tid<-taxID$ID[match(xml_attr(xml_find_all(x, "taxonomy"),"taxonomy-id"),taxID$ref)]
    return(taxData$name[taxData$ID%in%tid])
  }
})

keTaxWoe<-lapply(xml_find_all(xData, "/data/key-event/applicability"),FUN=function(x){
  if("taxonomy"%in%xml_name(xml_children(x))){
    twoe<-xml_text(xml_find_all(x, "taxonomy/evidence"))
    return(twoe)
  }
})


# proportion of KEs with taxonomic information
sum(!sapply(keTaxDom, is.null)) / length(keTaxDom) # 247 / 1118 = 0.2209302 :(

# Other KE data
keData<-data.frame(
  ID=keID$ID[match(xml_attr(xml_find_all(xData, "/data/key-event"), "id"),keID$ref)],
  title=xml_text(xml_find_all(xData, "/data/key-event/title")),
  LOBO=xml_text(xml_find_all(xData, "/data/key-event/biological-organization-level")),
  taxDom=I(keTaxDom),
  taxWoe=I(keTaxWoe),
  stringsAsFactors=FALSE
)


#### KEY EVENT RELATIONSHIP (KER) DATA TABLE ####

# KER taxonomy data (lists all species to which a KERs is assigned in the AOP Wiki and respective WoE calls)
kerTaxDom<-lapply(xml_find_all(xData, "/data/key-event-relationship/taxonomic-applicability"),FUN=function(x){
  if("taxonomy"%in%xml_name(xml_children(x))){
    tid<-taxID$ID[match(xml_attr(xml_find_all(x, "taxonomy"),"taxonomy-id"),taxID$ref)]
    return(taxData$name[taxData$ID%in%tid])
  }
})

kerTaxWoe<-lapply(xml_find_all(xData, "/data/key-event-relationship/taxonomic-applicability"),FUN=function(x){
  if("taxonomy"%in%xml_name(xml_children(x))){
    twoe<-xml_text(xml_find_all(x, "taxonomy/evidence"))
    return(twoe)
  }
})


# proportion of KERs with taxonomic information
sum(!sapply(kerTaxDom, is.null)) / length(kerTaxDom) # 230 / 1338 = 0.1718984

# all other KER Data
kerData<-data.frame(
  ID=kerID$ID[match(xml_attr(xml_find_all(xData, "/data/key-event-relationship"), "id"),kerID$ref)],
  KEup=keID$ID[match(xml_text(xml_find_all(xData, "/data/key-event-relationship/title/upstream-id")),keID$ref)],
  KEdown=keID$ID[match(xml_text(xml_find_all(xData, "/data/key-event-relationship/title/downstream-id")),keID$ref)],
  taxDom=I(kerTaxDom),
  taxWoe=I(kerTaxWoe),
  stringsAsFactors=FALSE
)

#### AOP DATA TABLE ####

# OECD status: not all AOPs have an "oecd-status" xml tag, so must use "if" to return NA when missing
oecdStatus<-sapply(xml_find_all(xData, "/data/aop/status"),FUN=function(x){
  if("oecd-status"%in%xml_name(xml_children(x))){
    return(xml_text(xml_find_all(x,"oecd-status")))
  }else{
    return("not specified")
  }
})

# SAAOP status: not all AOPs have an "saaop-status" xml tag, so must use "if" to return NA when missing
saaopStatus<-sapply(xml_find_all(xData, "/data/aop/status"),FUN=function(x){
  if("saaop-status"%in%xml_name(xml_children(x))){
    return(xml_text(xml_find_all(x,"saaop-status")))
  }else{
    return("not specified")
  }
})

# MIEs: more than one MIE possible per AOP, so must return list
mies<-lapply(xml_find_all(xData, "/data/aop"),FUN=function(x){
  if("molecular-initiating-event"%in%xml_name(xml_children(x))){
    return(keID$ID[match(xml_attr(xml_find_all(x, "molecular-initiating-event"),"key-event-id"),keID$ref)])
  }else{
    return(NULL)
  }
})

# AOs: more than one AO possible per AOP, so must return list
aos<-lapply(xml_find_all(xData, "/data/aop"),FUN=function(x){
  if("adverse-outcome"%in%xml_name(xml_children(x))){
    return(keID$ID[match(xml_attr(xml_find_all(x, "adverse-outcome"),"key-event-id"),keID$ref)])
  }else{
    return(NULL)
  }
})

# KEs: more than one KE possible per AOP, so must return list
kes<-lapply(xml_find_all(xData, "/data/aop/key-events"),FUN=function(x){
  if("key-event"%in%xml_name(xml_children(x))){
    return(keID$ID[match(xml_attr(xml_find_all(x, "key-event"),"id"),keID$ref)])
  }else{
    return(NULL)
  }
})

# KERs: more than one KER per AOP, each with AOP-specific "adjaceny", "quantitative understanding", and "WoE"
# So must return data frame of KERs
kers<-lapply(xml_find_all(xData, "/data/aop/key-event-relationships"),FUN=function(x){
  if("relationship"%in%xml_name(xml_children(x))){
    return(data.frame(
      ID=kerID$ID[match(xml_attr(xml_find_all(x, "relationship"),"id"),kerID$ref)],
      adjacency=xml_text(xml_find_all(x, "relationship/adjacency")),
      quant=xml_text(xml_find_all(x, "relationship/quantitative-understanding-value")),
      woe=xml_text(xml_find_all(x, "relationship/evidence")),
      stringsAsFactors=FALSE
    ))
  }else{
    return(NULL)
  }
})

# add KEs and MIE/AO designation (which is AOP-specific) for each KER in kers data.frame
for(i in 1:length(kers)){
  if(length(kers[[i]])>0){
    KEup<-kerData$KEup[match(kers[[i]]$ID,kerData$ID)]
    KEDup<-sapply(KEup, FUN=function(x){
      if(x%in%mies[[i]]){
        return("MIE")
      }else{
        if(x%in%aos[[i]]){
          return("AO")
        }else{
          return("KE")
        }
      }
    })
    
    KEdown<-kerData$KEdown[match(kers[[i]]$ID,kerData$ID)]
    KEDdown<-sapply(KEdown, FUN=function(x){
      if(x%in%mies[[i]]){
        return("MIE")
      }else{
        if(x%in%aos[[i]]){
          return("AO")
        }else{
          return("KE")
        }
      }
    })
    
    kers[[i]]<-data.frame(
      ID=kers[[i]]$ID,
      KEup=KEup,
      KEDup=KEDup,
      KEdown=KEdown,
      KEDdown=KEDdown,
      adjacency=kers[[i]]$adjacency,
      quant=kers[[i]]$quant,
      woe=kers[[i]]$woe,
      row.names=NULL,
      stringsAsFactors = FALSE
    )
  }
}

aopData<-data.frame(
  ID=aopID$ID[match(xml_attr(xml_find_all(xData, "/data/aop"), "id"),aopID$ref)],
  oecdStatus=oecdStatus,
  saaopStatus=saaopStatus,
  mies=I(mies),
  aos=I(aos),
  kes=I(kes),
  kers=I(kers),
  stringsAsFactors=FALSE
)


#### CREATE AOP NETWORK (iGraph Object) ####

## Currate AOPs that can be used in network

# remove all "archived" AOPs based on SAAOP status (and not NA)
aData<-aopData[!aopData$saaopStatus=="Archived",]

# remove all "empty" AOPs (i.e. AOPs that have ZERO MIEs, AOs, KEs, and/or ZERO KERs)
mieNull<-sapply(aData$mies, is.null)
aoNull<-sapply(aData$aos, is.null)
keNull<-sapply(aData$kes, is.null)
kerNull<-sapply(aData$kers, is.null)
emptyAOP<-(mieNull & aoNull & keNull) | kerNull
aData<-aData[!emptyAOP,]

## Create iGraph object

# Combine all KER tables from all AOPs
kerSet<-do.call("rbind", aData$kers) 

# Create edge list from unique "KEup" and "KEdown" pairs ("as.character" ensures the Nodes are named correctly when converting to iGraph object)
edgeList<-unique(kerSet[,c("ID","KEup","KEdown")])
edgeList$ID<-as.character(edgeList$ID)
edgeList$KEup<-as.character(edgeList$KEup)
edgeList$KEdown<-as.character(edgeList$KEdown)


# Create iGraph object from edgeList (edgelist must be a matrix)
g1<-graph_from_edgelist(as.matrix(edgeList[,c("KEup","KEdown")]), directed=TRUE)


#### MAP VERTEX (KE) ATTRIBUTES ####

# add "ID" attribute (same as "name")
V(g1)$ID<-V(g1)$name

# Add KE_KED (KED=Key event designation. "MIE", "KE", or "AO") based on "once an MIE/AO always an MIE/AO" (3 steps)
# step 1:combine KE/KED data from each AOP into one table
kedDat<-do.call("rbind", aData$kers) 

# step 2: combine all unique KE/ KEDs  into a single two column table
kedDat<-data.frame(KE=c(kedDat$KEup,kedDat$KEdown), KED=c(kedDat$KEDup, kedDat$KEDdown), stringsAsFactors=FALSE) 
kedDat<-unique(kedDat[order(as.numeric(kedDat$KE)),])

# step 3: assign KED to V(g1) based on table ("once an MIE/AO always an MIE/AO")
V(g1)$KE_KED<-sapply(V(g1)$ID, FUN=function(x){
  if("MIE"%in%kedDat$KED[kedDat$KE==x]){
    return("MIE")   
  }else{
    if("AO"%in%kedDat$KED[kedDat$KE==x]){
      return("AO")
    }else{
      return("KE")
    }
  }
})


# Add KE_PD (PD="Path Designation. path origin or path terminus)
g1<-add_KE_PD(g1)


# add colour attribute (MIE=green, KE=white, AOP=red)
V(g1)$col<-"white"
V(g1)$col[V(g1)$KE_KED=="MIE"]<-"green"
V(g1)$col[V(g1)$KE_KED=="AO"]<-"red"


# Add AOP IDs. Note: Each KE may belong to more than one AOP, so the AOP_IDs object is a list not just a single AOP ID
V(g1)$AOP_ID<-list(vector())
for(i in 1:nrow(aData)){
  keAll<-c(aData$mies[[i]], aData$aos[[i]], aData$kes[[i]])
  if(length(keAll)>0){
    for(j in keAll){
      if(j%in%V(g1)$ID){
        V(g1)$AOP_ID[[match(j,V(g1)$ID)]]<-c(V(g1)$AOP_ID[[match(j,V(g1)$ID)]],aData$ID[i])
      }
    }
  }
}

# map  level of biological organization (LOBO)
V(g1)$LOBO<-keData$LOBO[match(V(g1)$ID, keData$ID)]

# map titles as KE attribute
V(g1)$title<-keData$title[match(V(g1)$ID, keData$ID)]


# map taxonomic domains 
V(g1)$taxDom<-keData$taxDom[match(V(g1)$ID, keData$ID)]
V(g1)$taxWoe<-keData$taxWoe[match(V(g1)$ID, keData$ID)]


#### MAP EDGE (KER) ATRTIBUTeS ####

# add KER ID as edge attribute
E(g1)$ID<-edgeList$ID

# Add AOP IDs. Note: Each KER may belong to more than one AOP, so the AOP_IDs object is a list not just a single AOP ID
E(g1)$AOP_ID<-list(vector())
for(i in 1:nrow(aData)){
  if(!is.null(aData$kers[[i]])){
    for(j in aData$kers[[i]]$ID){
      if(j%in%E(g1)$ID){
        E(g1)$AOP_ID[[match(j,E(g1)$ID)]]<-c(E(g1)$AOP_ID[[match(j,E(g1)$ID)]],aData$ID[i])
      }
    }
  }
}

# WoE and quantitative understanding

# currently WOE and Quant are AOP-SPECIFIC
# For this study, we will use the LOWEST WoE and Quant assigned to a KER, if it has multiple values

#WoE
woeList<-unique(kerSet[,c("ID","KEup","KEdown", "woe")])
dupIDs<-unique(woeList$ID[duplicated(woeList$ID)])
w<-vector()
for(i in dupIDs){
  kerW<-woeList$woe[woeList$ID==i]
  if("Low"%in%kerW){
    w<-c(w, "Low")
  }else{
    if("Moderate"%in%kerW){
      w<-c(w, "Moderate")
    }else{
      if("High"%in%kerW){
        w<-c(w,"High")
      }else{
        w<-c(w,"Not Specified")
      }
    }
  }
}
# remove duplicates
woeList<-woeList[!duplicated(woeList$ID),]
# add woes for duplicates
woeList$woe[match(dupIDs, woeList$ID)]<-w
# assign as edge attribute
E(g1)$woe[match(woeList$ID, E(g1)$ID)]<-woeList$woe

#convert to numeric score
wScores<-data.frame(w=c("High","Moderate","Low","Not Specified"), score=c(3, 2, 1, 1))
E(g1)$woe_score<-wScores$score[match(E(g1)$woe, wScores$w)]


# quant
quantList<-unique(kerSet[,c("ID","KEup","KEdown", "quant")])
dupIDs<-unique(quantList$ID[duplicated(quantList$ID)])
q<-vector()
for(i in dupIDs){
  kerQ<-quantList$quant[quantList$ID==i]
  if("Low"%in%kerQ){
    q<-c(q, "Low")
  }else{
    if("Moderate"%in%kerQ){
      q<-c(q, "Moderate")
    }else{
      if("High"%in%kerQ){
        q<-c(q,"High")
      }else{
        q<-c(q,"Not Specified")
      }
    }
  }
}
# remove duplicates
quantList<-quantList[!duplicated(quantList$ID),]
# add quants for duplicates
quantList$quant[match(dupIDs, quantList$ID)]<-q
# assign as edge attribute
E(g1)$quant[match(quantList$ID, E(g1)$ID)]<-quantList$quant

#convert to numeric score
qScores<-data.frame(w=c("High","Moderate","Low","Not Specified"), score=c(3, 2, 1, 1))
E(g1)$quant_score<-wScores$score[match(E(g1)$quant, wScores$w)]

# add taxanomic domain
E(g1)$taxDom<-kerData$taxDom[match(E(g1)$ID, kerData$ID)]
E(g1)$taxWoe<-kerData$taxWoe[match(E(g1)$ID, kerData$ID)]



#### SUBGRAPG OF AOPS OF INTEREST ####

select_KEs<-which(sapply(V(g1)$AOP_ID, function(x){
  any(x%in%unlist(AOPs_of_interest))
}))
aopNet<-induced_subgraph(g1, select_KEs)

# identify Adjacent and Non-Adjacent KERs #
aopNet<-add_KER_adjacency(aopNet)


#only endorsed AOPs
oecdAOPs<-AOPs_of_interest[aopData$oecdStatus[match(AOPs_of_interest, aopData$ID)]=="WPHA/WNT Endorsed"]

select_KEs<-which(sapply(V(g1)$AOP_ID, function(x){
  any(x%in%unlist(oecdAOPs))
}))
oecdNet<-induced_subgraph(g1, select_KEs)
oecdNet<-add_KER_adjacency(oecdNet)

g<-aopNet



#### SPECIES FITLERS/LAYERS #####
allSpeciesKE<-unique(unlist(V(g)$taxDom))
allSpeciesKER<-unique(unlist(E(g)$taxDom))
allSpecies<-base::union(allSpeciesKE, allSpeciesKER)
allSpecies<-allSpecies[order(allSpecies)]
# problem #1: there are mores species defined in the KEs than the KERs
# problem #2: different "names" for the same species (e.g. "human" vs "humans" vs "homo sapiens")
# specifically introduce species tags 

speciesKEs<-lapply(allSpecies, function(x){
  sapply(V(g)$taxDom, function(y){
    x%in%unlist(y)
  })
})
names(speciesKEs)<-allSpecies

speciesKERs<-lapply(allSpecies, function(x){
  sapply(E(g)$taxDom, function(y){
    x%in%unlist(y)
  })
})
names(speciesKERs)<-allSpecies
write.table (allSpecies, "allspecies.txt")

## Species tag for filters and layers ##
# Humans
species_of_interest_human<- c("all species", "human", "humans", "Homo sapiens")
# Mammals
species_of_interest_mammal<- c("all species", "human", "humans", "Homo sapiens", "mouse", "mice", "rat", "rats", "Pig", "pigs", "cat", "Ovis orientalis aries", "Bos taurus", "Canis lupus familiaris", "Erinaceus europaeus", "Pan troglodytes", "Rattus norvegicus", "Sprague-Dawley", "Bubalus bubalis", "Mus musculus", "Ovis aries", "Rattus sp.", "Sus scrofa", "Macaca mulatta", "Monodelphis domestica", "Pan troglodytes")
# Fish, AH removed pimephales promelas from LV's listing
species_of_interest_fish<- c("all species", "teleost fish", "Danio rerio","fathead minnow","Fundulus heteroclitus","gilthead bream","killifish","Oreochromis niloticus","Oryzias latipes","teleost fish","zebra fish","zebrafish", "Salmo salar", "Dicentrarchus labrax", "Perca flavescens", "bluefin tuna", "rainbow trout")
# Amphibians
species_of_interest_amphibian<- c("all species", "African clawed frog","Xenopus (Silurana) n. sp. tetraploid-1","Xenopus laevis","Xenopus laevis laevis", "Xenopus (Silurana) epitropicalis", "Xenopus tropicalis")
# Birds
species_of_interest_bird<- c("all species", "Meleagris gallopavo", "Gallus gallus", "chicken")

## MAMMALS 

soi_keTable_mammal<-sapply(species_of_interest_mammal, function(x) speciesKEs[[x]])
soi_kerTable_mammal<-sapply(species_of_interest_mammal, function(x) speciesKERs[[x]])

V(g)$v_taxDom_tag_mammal<-apply(soi_keTable_mammal, 1, any)
E(g)$e_taxDom_tag_mammal<-apply(soi_kerTable_mammal, 1, any)


# Taxanomic WoE
# if species_of_interest has MORE THAN ONE species, use the LOWEST WOE value
taxWoe_option<-2


# function to select "lowest" WoE value
low_mod_high_reduce<-function(x){
  if(any(x=="Low", na.rm=TRUE)){
    return("Low")
  }else if(any(x=="Moderate", na.rm=TRUE)){
    return("Moderate")
  }else if(any(x=="High", na.rm=TRUE)){
    return("High")
  }else{
    return(NA)
  }
}  
  

# KE WoE: retrieve species-specific KE TaxWoEs 
soi_KE_loc_mammal<-lapply(V(g)$taxDom, function(x) which(x%in%species_of_interest_mammal))
soi_KE_filtered_mammal<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KE_loc_mammal)){
    if(length(soi_KE_loc_mammal[[i]])==0){
      soi_KE_filtered_mammal[[i]]<-NA
    }else{
      soi_KE_filtered_mammal[[i]]<-low_mod_high_reduce(V(g)$taxWoe[[i]][soi_KE_loc_mammal[[i]]])
    }
  }
}

# KER WoE: retrieve species-specific KER TaxWoEs 
soi_KER_loc_mammal<-lapply(E(g)$taxDom, function(x) which(x%in%species_of_interest_mammal))
soi_KER_filtered_mammal<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KER_loc_mammal)){
    if(length(soi_KER_loc_mammal[[i]])==0){
      soi_KER_filtered_mammal[[i]]<-NA
    }else{
      soi_KER_filtered_mammal[[i]]<-low_mod_high_reduce(E(g)$taxWoe[[i]][soi_KER_loc_mammal[[i]]])
    }
  }
}


###FISH

soi_keTable_fish<-sapply(species_of_interest_fish,function(x) speciesKEs[[x]])
soi_kerTable_fish<-sapply(species_of_interest_fish, function(x) speciesKERs[[x]])

V(g)$v_taxDom_tag_fish<-apply(soi_keTable_fish,1, any)
E(g)$e_taxDom_tag_fish<-apply(soi_kerTable_fish, 1, any)


# Taxanomic WOE
# if species_of_interest has MORE THAN ONE species, use the LOWEST WOE value
taxWoe_option<-2


# function to select "lowest" WOE value
low_mod_high_reduce<-function(x){
  if(any(x=="Low", na.rm=TRUE)){
    return("Low")
  }else if(any(x=="Moderate", na.rm=TRUE)){
    return("Moderate")
  }else if(any(x=="High", na.rm=TRUE)){
    return("High")
  }else{
    return(NA)
  }
}  


# KE WOE: retrieve species-specific KE TaxWoEs 
soi_KE_loc_fish<-lapply(V(g)$taxDom, function(x) which(x%in%species_of_interest_fish))
soi_KE_filtered_fish<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KE_loc_fish)){
    if(length(soi_KE_loc_fish[[i]])==0){
      soi_KE_filtered_fish[[i]]<-NA
    }else{
      soi_KE_filtered_fish[[i]]<-low_mod_high_reduce(V(g)$taxWoe[[i]][soi_KE_loc_fish[[i]]])
    }
  }
}

# KER WOE: retrieve species-specific KER TaxWoes 
soi_KER_loc_fish<-lapply(E(g)$taxDom, function(x) which(x%in%species_of_interest_fish))
soi_KER_filtered_fish<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KER_loc_fish)){
    if(length(soi_KER_loc_fish[[i]])==0){
      soi_KER_filtered_fish[[i]]<-NA
    }else{
      soi_KER_filtered_fish[[i]]<-low_mod_high_reduce(E(g)$taxWoe[[i]][soi_KER_loc_fish[[i]]])
    }
  }
}


###AMPHIBIAN

soi_keTable_amphibian<-sapply(species_of_interest_amphibian, function(x) speciesKEs[[x]])
soi_kerTable_amphibian<-sapply(species_of_interest_amphibian, function(x) speciesKERs[[x]])

V(g)$v_taxDom_tag_amphibian<-apply(soi_keTable_amphibian, 1, any)
E(g)$e_taxDom_tag_amphibian<-apply(soi_kerTable_amphibian, 1, any)


# Taxanomic WOE
# if species_of_interest has MORE THAN ONE species, use the LOWEST WOE value
taxWoe_option<-2


# function to select "lowest" WOE value
low_mod_high_reduce<-function(x){
  if(any(x=="Low", na.rm=TRUE)){
    return("Low")
  }else if(any(x=="Moderate", na.rm=TRUE)){
    return("Moderate")
  }else if(any(x=="High", na.rm=TRUE)){
    return("High")
  }else{
    return(NA)
  }
}  


# KE WoE: retrieve species-specific KE TaxWoes 
soi_KE_loc_amphibian<-lapply(V(g)$taxDom, function(x) which(x%in%species_of_interest_amphibian))
soi_KE_filtered_amphibian<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KE_loc_amphibian)){
    if(length(soi_KE_loc_amphibian[[i]])==0){
      soi_KE_filtered_amphibian[[i]]<-NA
    }else{
      soi_KE_filtered_amphibian[[i]]<-low_mod_high_reduce(V(g)$taxWoe[[i]][soi_KE_loc_amphibian[[i]]])
    }
  }
}

# KER WoE: retrieve species-specific KER TaxWoes 
soi_KER_loc_amphibian<-lapply(E(g)$taxDom, function(x) which(x%in%species_of_interest_amphibian))
soi_KER_filtered_amphibian<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KER_loc_amphibian)){
    if(length(soi_KER_loc_amphibian[[i]])==0){
      soi_KER_filtered_amphibian[[i]]<-NA
    }else{
      soi_KER_filtered_amphibian[[i]]<-low_mod_high_reduce(E(g)$taxWoe[[i]][soi_KER_loc_amphibian[[i]]])
    }
  }
}


###HUMAN

soi_keTable_human<-sapply(species_of_interest_human, function(x) speciesKEs[[x]])
soi_kerTable_human<-sapply(species_of_interest_human, function(x) speciesKERs[[x]])

V(g)$v_taxDom_tag_human<-apply(soi_keTable_human, 1, any)
E(g)$e_taxDom_tag_human<-apply(soi_kerTable_human, 1, any)


# Taxanomic WoE
# if species_of_interest has MORE THAN ONE species, use the LOWEST WoE value
taxWoe_option<-2


# function to select "lowest" WOE value
low_mod_high_reduce<-function(x){
  if(any(x=="Low", na.rm=TRUE)){
    return("Low")
  }else if(any(x=="Moderate", na.rm=TRUE)){
    return("Moderate")
  }else if(any(x=="High", na.rm=TRUE)){
    return("High")
  }else{
    return(NA)
  }
}  


# KE WoE: retrieve species-specific KE TaxWoes 
soi_KE_loc_human<-lapply(V(g)$taxDom, function(x) which(x%in%species_of_interest_human))
soi_KE_filtered_human<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KE_loc_human)){
    if(length(soi_KE_loc_human[[i]])==0){
      soi_KE_filtered_human[[i]]<-NA
    }else{
      soi_KE_filtered_human[[i]]<-low_mod_high_reduce(V(g)$taxWoe[[i]][soi_KE_loc_human[[i]]])
    }
  }
}

# KER WoE: retrieve species-specific KER TaxWoes 
soi_KER_loc_human<-lapply(E(g)$taxDom, function(x) which(x%in%species_of_interest_human))
soi_KER_filtered_human<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KER_loc_human)){
    if(length(soi_KER_loc_human[[i]])==0){
      soi_KER_filtered_human[[i]]<-NA
    }else{
      soi_KER_filtered_human[[i]]<-low_mod_high_reduce(E(g)$taxWoe[[i]][soi_KER_loc_human[[i]]])
    }
  }
}


##BIRDS

soi_keTable_bird<-sapply(species_of_interest_bird, function(x) speciesKEs[[x]])
soi_kerTable_bird<-sapply(species_of_interest_bird, function(x) speciesKERs[[x]])

V(g)$v_taxDom_tag_bird<-apply(soi_keTable_bird, 1, any)
E(g)$e_taxDom_tag_bird<-apply(soi_kerTable_bird, 1, any)


# Taxanomic WoE
# if species_of_interest has MORE THAN ONE species, use the LOWEST WOE value
taxWoe_option<-2


# function to select "lowest" WoE value
low_mod_high_reduce<-function(x){
  if(any(x=="Low", na.rm=TRUE)){
    return("Low")
  }else if(any(x=="Moderate", na.rm=TRUE)){
    return("Moderate")
  }else if(any(x=="High", na.rm=TRUE)){
    return("High")
  }else{
    return(NA)
  }
}  


# KE WoE: retrieve species-specific KE TaxWoes 
soi_KE_loc_bird<-lapply(V(g)$taxDom, function(x) which(x%in%species_of_interest_bird))
soi_KE_filtered_bird<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KE_loc_bird)){
    if(length(soi_KE_loc_bird[[i]])==0){
      soi_KE_filtered_bird[[i]]<-NA
    }else{
      soi_KE_filtered_bird[[i]]<-low_mod_high_reduce(V(g)$taxWoe[[i]][soi_KE_loc_bird[[i]]])
    }
  }
}

# KER WoE: retrieve species-specific KER TaxWoes 
soi_KER_loc_bird<-lapply(E(g)$taxDom, function(x) which(x%in%species_of_interest_bird))
soi_KER_filtered_bird<-list()

if(taxWoe_option==2){
  for(i in 1:length(soi_KER_loc_bird)){
    if(length(soi_KER_loc_bird[[i]])==0){
      soi_KER_filtered_bird[[i]]<-NA
    }else{
      soi_KER_filtered_bird[[i]]<-low_mod_high_reduce(E(g)$taxWoe[[i]][soi_KER_loc_bird[[i]]])
    }
  }
}


#### EMERGENT AOPS ####

### First need to identify user defined linear AOPs
aoiData<-aData[aData$ID%in%AOPs_of_interest,]

uLaops<-list()
uLaops_adj<-list()

uLaopCount<-vector()
uLaopCount_adj<-vector()

for(i in 1:nrow(aoiData)){

  # edge list for each user-define AOP
  eList<-aoiData$kers[[i]]
  eList$KEup<-as.character(eList$KEup)
  eList$KEdown<-as.character(eList$KEdown)
  
  # subgraph by edgelist
  subG<-graph_from_edgelist(as.matrix(eList[,c("KEup", "KEdown")]), directed=TRUE)
  
  # add KE_KED to each KE (MIE, AO, or KE)
  subKed<-data.frame(KE=c(eList$KEup,eList$KEdown), KED=c(eList$KEDup, eList$KEDdown), stringsAsFactors=FALSE) 
  subKed<-unique(subKed)
  V(subG)$KE_KED<-subKed$KED[match(V(subG)$name,subKed$KE)]
  
  # determine adjacency (algorithmically)
  subG<-add_KER_adjacency(subG)
  subG_adj<-subgraph.edges(subG, eids=E(subG)[E(subG)$adjacency=="adjacent"] )
  
  # ALL linear AOPs (including paths with non-adjacent KERs)
  uLaops[[i]]<-linear.AOPs(subG)
  if(length(uLaops[[i]])==0){
    uLaopCount<-c(uLaopCount,0)
  }else{
    uLaopCount<-c(uLaopCount,sum(sapply(uLaops[[i]],length)))
  }
  
  # Linear AOPs with adjacent KERs only
  uLaops_adj[[i]]<-linear.AOPs(subG_adj)
  if(length(uLaops_adj[[i]])==0){
    uLaopCount_adj<-c(uLaopCount_adj,0)
  }else{
    uLaopCount_adj<-c(uLaopCount_adj,sum(sapply(uLaops_adj[[i]],length)))
  }
}

sum(uLaopCount)
sum(uLaopCount_adj)


### Determine ALL linear AOPs in full network, and adjacent only network

# full network
netLaops<-linear.AOPs(g)
netLaopCount<-sapply(netLaops, length)
sum(netLaopCount)

# adjecent-KER-only network
g_adj<-subgraph.edges(g, eids=E(g)[E(g)$adjacency=="adjacent"] )
netLaops_adj<-linear.AOPs(g_adj)
netLaopCount_adj<-sapply(netLaops_adj, length)
sum(netLaopCount_adj)


### Compare user-defined vs total LAOPS to identify "emergent" AOP (adjacent KERs only)

# user defined KE paths
uLaopKEs<-sapply(uLaops_adj, function(x){
  sapply(x, function(y){
    sapply(y, function(z){
      paste(attr(z,"names"),collapse=", ")
    })
  })
})
uLaopTable<-unlist(uLaopKEs, use.names = FALSE)

# all KE paths in the network
netLaopKEs<-sapply(netLaops_adj, function(x){
  sapply(x, function(y){
    paste(attr(y,"names"),collapse=", ")
  })
})
netLaopTable<-unlist(netLaopKEs, use.names = FALSE)

# combine into a table and identify "emergent" AOPs
allKEPathTable<-data.frame(emergent=!netLaopTable%in%uLaopTable, path=netLaopTable, stringsAsFactors = FALSE)
mie_ao<-lapply(allKEPathTable[,2], function(x){
  s<-unlist(strsplit(x, ", "))
  return(c(s[1],s[length(s)]))
})
mie_ao<-do.call(rbind, mie_ao)

emergent_table<-data.frame(emergent=allKEPathTable$emergent, mie=mie_ao[,1], ao=mie_ao[,2], path=allKEPathTable$path, stringsAsFactors = FALSE)

write.table(emergent_table, "emergent_adjacent_mie_to_ao_paths.txt", sep="\t", row.names = FALSE)


#### EXPORT IGRAPH OBJECT TO CYTOSCAPE ####
# open Cytoscape before running command 
createNetworkFromIgraph(g, "thyroidNet_all_tags")

